@@ -76,7 +76,7 @@ QFilePrivate::openExternalFile(QIODevice::OpenMode flags, FILE *fh, QFile::FileH
 QAbstractFileEngine *QFilePrivate::engine() const
 {
     if (!fileEngine)
-        fileEngine.reset(QAbstractFileEngine::create(fileName));
+        fileEngine = QAbstractFileEngine::create(fileName);
     return fileEngine.get();
 }
 
@@ -123,7 +123,7 @@ QAbstractFileEngine *QFilePrivate::engine() const
 
     \snippet file/file.cpp 0
 
-    The QIODevice::Text flag passed to open() tells Qt to convert
+    The \l{QIODeviceBase::}{Text} flag passed to open() tells Qt to convert
     Windows-style line terminators ("\\r\\n") into C++-style
     terminators ("\\n"). By default, QFile assumes binary, i.e. it
     doesn't perform any conversion on the bytes stored in the file.
@@ -269,10 +269,10 @@ QFile::~QFile()
 }
 
 /*!
-    Returns the name set by setFileName() or to the QFile
-    constructors.
+    Returns the name of the file as set by setFileName(), rename(), or
+    by the QFile constructors.
 
-    \sa setFileName(), QFileInfo::fileName()
+    \sa setFileName(), rename(), QFileInfo::fileName()
 */
 QString QFile::fileName() const
 {
@@ -452,6 +452,22 @@ QFile::remove(const QString &fileName)
     return QFile(fileName).remove();
 }
 
+/*!
+    \since 6.9
+
+    Returns \c true if Qt supports moving files to a trash (recycle bin) in the
+    current operating system using the moveToTrash() function, \c false
+    otherwise. Note that this function returning \c true does not imply
+    moveToTrash() will succeed. In particular, this function does not check if
+    the user has disabled the functionality in their settings.
+
+    \sa moveToTrash()
+*/
+bool QFile::supportsMoveToTrash()
+{
+    return QFileSystemEngine::supportsMoveFileToTrash();
+}
+
 /*!
     \since 5.15
 
@@ -462,7 +478,9 @@ QFile::remove(const QString &fileName)
 //! [move-to-trash-common]
     The time for this function to run is independent of the size of the file
     being trashed. If this function is called on a directory, it may be
-    proportional to the number of files being trashed.
+    proportional to the number of files being trashed. If the current
+    fileName() points to a symbolic link, this function will move the link to
+    the trash, possibly breaking it, not the target of the link.
 
     This function uses the Windows and \macos APIs to perform the trashing on
     those two operating systems. Elsewhere (Unix systems), this function
@@ -477,9 +495,12 @@ QFile::remove(const QString &fileName)
     themselves mount points).
 //! [move-to-trash-common]
 
-    \note On systems where the system API doesn't report the location of the file in the
-    trash, fileName() will be set to the null string once the file has been moved. On
-    systems that don't have a trash can, this function always returns false.
+    \note On systems where the system API doesn't report the location of the
+    file in the trash, fileName() will be set to the null string once the file
+    has been moved. On systems that don't have a trash can, this function
+    always returns \c false (see supportsMoveToTrash()).
+
+    \sa supportsMoveToTrash(), remove(), QDir::remove()
 */
 bool
 QFile::moveToTrash()
@@ -510,7 +531,7 @@ QFile::moveToTrash()
     \since 5.15
     \overload
 
-    Moves the file specified by fileName() to the trash. Returns \c true if successful,
+    Moves the file specified by \a fileName to the trash. Returns \c true if successful,
     and sets \a pathInTrash (if provided) to the path at which the file can be found within
     the trash; otherwise returns \c false.
 
@@ -584,7 +605,7 @@ QFile::rename(const QString &newName)
             return false;
         }
 
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_LINUX) && QT_CONFIG(temporaryfile)
         // rename() on Linux simply does nothing when renaming "foo" to "Foo" on a case-insensitive
         // FS, such as FAT32. Move the file away and rename in 2 steps to work around.
         QTemporaryFileName tfn(d->fileName);
@@ -633,14 +654,21 @@ QFile::rename(const QString &newName)
             return true;
         }
 
+        // Engine was unable to rename and the fallback will delete the original file,
+        // so we have to back out here on case-insensitive file systems:
+        if (changingCase) {
+            d->setError(QFile::RenameError, d->fileEngine->errorString());
+            return false;
+        }
+
         if (isSequential()) {
             d->setError(QFile::RenameError, tr("Will not rename sequential file using block copy"));
             return false;
         }
 
         QFile out(newName);
-        if (open(QIODevice::ReadOnly)) {
-            if (out.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+        if (open(QIODevice::ReadOnly | QIODevice::Unbuffered)) {
+            if (out.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Unbuffered)) {
                 bool error = false;
                 char block[4096];
                 qint64 bytes;
@@ -784,12 +812,12 @@ QFile::copy(const QString &newName)
             return true;
         } else {
             bool error = false;
-            if (!open(QFile::ReadOnly)) {
+            if (!open(QFile::ReadOnly | QFile::Unbuffered)) {
                 error = true;
                 d->setError(QFile::CopyError, tr("Cannot open %1 for input").arg(d->fileName));
             } else {
                 const auto fileTemplate = "%1/qt_temp.XXXXXX"_L1;
-#ifdef QT_NO_TEMPORARYFILE
+#if !QT_CONFIG(temporaryfile)
                 QFile out(fileTemplate.arg(QFileInfo(newName).path()));
                 if (!out.open(QIODevice::ReadWrite))
                     error = true;
@@ -809,6 +837,7 @@ QFile::copy(const QString &newName)
                     if (!d->engine()->cloneTo(out.d_func()->engine())) {
                         char block[4096];
                         qint64 totalRead = 0;
+                        out.setOpenMode(ReadWrite | Unbuffered);
                         while (!atEnd()) {
                             qint64 in = read(block, sizeof(block));
                             if (in <= 0)
@@ -841,7 +870,7 @@ QFile::copy(const QString &newName)
                                         .arg(newName, out.errorString()));
                         }
                     }
-#ifdef QT_NO_TEMPORARYFILE
+#if !QT_CONFIG(temporaryfile)
                     if (error)
                         out.remove();
 #else
@@ -881,14 +910,14 @@ QFile::copy(const QString &fileName, const QString &newName)
 }
 
 /*!
-    Opens the file using OpenMode \a mode, returning true if successful;
-    otherwise false.
+    Opens the file using \a mode flags, returning \c true if successful;
+    otherwise returns \c false.
 
-    The \a mode must be QIODevice::ReadOnly, QIODevice::WriteOnly, or
-    QIODevice::ReadWrite. It may also have additional flags, such as
-    QIODevice::Text and QIODevice::Unbuffered.
+    The flags for \a mode must include \l QIODeviceBase::ReadOnly,
+    \l WriteOnly, or \l ReadWrite. It may also have additional flags,
+    such as \l Text and \l Unbuffered.
 
-    \note In \l{QIODevice::}{WriteOnly} or \l{QIODevice::}{ReadWrite}
+    \note In \l{WriteOnly} or \l{ReadWrite}
     mode, if the relevant file does not already exist, this function
     will try to create a new file before opening it. The file will be
     created with mode 0666 masked by the umask on POSIX systems, and
@@ -897,7 +926,7 @@ QFile::copy(const QString &fileName, const QString &newName)
     of the file name, otherwise, it won't be possible to create this
     non-existing file.
 
-    \sa QIODevice::OpenMode, setFileName()
+    \sa QT_USE_NODISCARD_FILE_OPEN, setFileName()
 */
 bool QFile::open(OpenMode mode)
 {
@@ -941,7 +970,7 @@ bool QFile::open(OpenMode mode)
     such permissions will generate warnings when the Security tab of the Properties dialog
     is opened. Granting the group all permissions granted to others avoids such warnings.
 
-    \sa QIODevice::OpenMode, setFileName()
+    \sa QIODevice::OpenMode, setFileName(), QT_USE_NODISCARD_FILE_OPEN
     \since 6.3
 */
 bool QFile::open(OpenMode mode, QFile::Permissions permissions)
@@ -998,7 +1027,7 @@ bool QFile::open(OpenMode mode, QFile::Permissions permissions)
            you cannot use this QFile with a QFileInfo.
     \endlist
 
-    \sa close()
+    \sa close(), QT_USE_NODISCARD_FILE_OPEN
 
     \b{Note for the Windows Platform}
 
@@ -1064,7 +1093,7 @@ bool QFile::open(FILE *fh, OpenMode mode, FileHandleFlags handleFlags)
     \warning Since this function opens the file without specifying the file name,
              you cannot use this QFile with a QFileInfo.
 
-    \sa close()
+    \sa close(), QT_USE_NODISCARD_FILE_OPEN
 */
 bool QFile::open(int fd, OpenMode mode, FileHandleFlags handleFlags)
 {
