@@ -37,7 +37,10 @@
 #define SECURITY_WIN32
 #include <security.h>
 
+#include <cstdio>
+
 #include <QtCore/private/qfunctions_win_p.h>
+#include <QtCore/private/wcharhelpers_win_p.h>
 
 #ifndef SPI_GETPLATFORMTYPE
 #define SPI_GETPLATFORMTYPE 257
@@ -682,8 +685,7 @@ static QString readSymLink(const QFileSystemEntry &link)
             DWORD len;
             wchar_t buffer[MAX_PATH];
             const QString volumeName = "\\\\?\\"_L1 + matchVolume.captured();
-            if (GetVolumePathNamesForVolumeName(reinterpret_cast<LPCWSTR>(volumeName.utf16()),
-                                                buffer, MAX_PATH, &len)
+            if (GetVolumePathNamesForVolumeName(qt_castToWchar(volumeName), buffer, MAX_PATH, &len)
                 != 0) {
                 result.replace(0, matchVolume.capturedLength(), QString::fromWCharArray(buffer));
             }
@@ -1010,10 +1012,10 @@ static inline QByteArray fileId(HANDLE handle)
     BY_HANDLE_FILE_INFORMATION info;
     if (GetFileInformationByHandle(handle, &info)) {
         char buffer[sizeof "01234567:0123456701234567"];
-        qsnprintf(buffer, sizeof(buffer), "%lx:%08lx%08lx",
-                  info.dwVolumeSerialNumber,
-                  info.nFileIndexHigh,
-                  info.nFileIndexLow);
+        std::snprintf(buffer, sizeof(buffer), "%lx:%08lx%08lx",
+                      info.dwVolumeSerialNumber,
+                      info.nFileIndexHigh,
+                      info.nFileIndexLow);
         return buffer;
     }
     return QByteArray();
@@ -1069,7 +1071,7 @@ QByteArray QFileSystemEngine::id(HANDLE fHandle)
 
 //static
 bool QFileSystemEngine::setFileTime(HANDLE fHandle, const QDateTime &newDate,
-                                    QAbstractFileEngine::FileTime time, QSystemError &error)
+                                    QFile::FileTime time, QSystemError &error)
 {
     FILETIME fTime;
     FILETIME *pLastWrite = nullptr;
@@ -1077,15 +1079,15 @@ bool QFileSystemEngine::setFileTime(HANDLE fHandle, const QDateTime &newDate,
     FILETIME *pCreationTime = nullptr;
 
     switch (time) {
-    case QAbstractFileEngine::ModificationTime:
+    case QFile::FileModificationTime:
         pLastWrite = &fTime;
         break;
 
-    case QAbstractFileEngine::AccessTime:
+    case QFile::FileAccessTime:
         pLastAccess = &fTime;
         break;
 
-    case QAbstractFileEngine::BirthTime:
+    case QFile::FileBirthTime:
         pCreationTime = &fTime;
         break;
 
@@ -1388,7 +1390,7 @@ bool QFileSystemEngine::fillMetaData(const QFileSystemEntry &entry, QFileSystemM
     // Check for ".lnk": Directories named ".lnk" should be skipped, corrupted
     // link files should still be detected as links.
     const QString origFilePath = entry.filePath();
-    if (origFilePath.endsWith(".lnk"_L1) && !isDirPath(origFilePath, nullptr)) {
+    if (bool exists; origFilePath.endsWith(".lnk"_L1) && !isDirPath(origFilePath, &exists) && exists) {
         data.entryFlags |= QFileSystemMetaData::WinLnkType;
         fname = QFileSystemEntry(readLink(entry));
     } else {
@@ -1524,58 +1526,74 @@ static bool createDirectoryWithParents(const QString &nativeName,
     return isDir(nativeName);
 }
 
-//static
-bool QFileSystemEngine::createDirectory(const QFileSystemEntry &entry, bool createParents,
-                                        std::optional<QFile::Permissions> permissions)
+bool QFileSystemEngine::mkpath(const QFileSystemEntry &entry,
+                               std::optional<QFile::Permissions> permissions)
 {
     QString dirName = entry.filePath();
     Q_CHECK_FILE_NAME(dirName, false);
 
-    dirName = QDir::toNativeSeparators(QDir::cleanPath(dirName));
-
     QNativeFilePermissions nativePermissions(permissions, true);
     if (!nativePermissions.isOk())
         return false;
 
     auto securityAttributes = nativePermissions.securityAttributes();
+    dirName = QDir::toNativeSeparators(QDir::cleanPath(dirName));
 
     // try to mkdir this directory
     DWORD lastError;
     if (mkDir(dirName, securityAttributes, &lastError))
         return true;
-    // mkpath should return true, if the directory already exists, mkdir false.
-    if (!createParents)
-        return false;
+    // mkpath should return true, if the directory already exists
     if (lastError == ERROR_ALREADY_EXISTS || lastError == ERROR_ACCESS_DENIED)
         return isDirPath(dirName, nullptr);
 
     return createDirectoryWithParents(dirName, securityAttributes, false);
 }
 
-//static
-bool QFileSystemEngine::removeDirectory(const QFileSystemEntry &entry, bool removeEmptyParents)
+bool QFileSystemEngine::mkdir(const QFileSystemEntry &entry,
+                              std::optional<QFile::Permissions> permissions)
 {
     QString dirName = entry.filePath();
     Q_CHECK_FILE_NAME(dirName, false);
 
-    if (removeEmptyParents) {
-        dirName = QDir::toNativeSeparators(QDir::cleanPath(dirName));
-        for (int oldslash = 0, slash=dirName.length(); slash > 0; oldslash = slash) {
-            const auto chunkRef = QStringView{dirName}.left(slash);
-            if (chunkRef.length() == 2 && chunkRef.at(0).isLetter()
-                && chunkRef.at(1) == u':') {
-                break;
-            }
-            const QString chunk = chunkRef.toString();
-            if (!isDirPath(chunk, nullptr))
-                return false;
-            if (!rmDir(chunk))
-                return oldslash != 0;
-            slash = dirName.lastIndexOf(QDir::separator(), oldslash-1);
+
+    QNativeFilePermissions nativePermissions(permissions, true);
+    if (!nativePermissions.isOk())
+        return false;
+
+    dirName = QDir::toNativeSeparators(QDir::cleanPath(dirName));
+    return mkDir(dirName, nativePermissions.securityAttributes());
+}
+
+bool QFileSystemEngine::rmdir(const QFileSystemEntry &entry)
+{
+    QString dirName = entry.filePath();
+    Q_CHECK_FILE_NAME(dirName, false);
+
+    return rmDir(dirName);
+}
+
+bool QFileSystemEngine::rmpath(const QFileSystemEntry &entry)
+{
+    const QString dirName = QDir::toNativeSeparators(QDir::cleanPath(entry.filePath()));
+    Q_CHECK_FILE_NAME(dirName, false);
+
+    for (int oldslash = 0, slash = dirName.size(); slash > 0; oldslash = slash) {
+        const auto chunkRef = QStringView{dirName}.left(slash);
+        if (chunkRef.length() == 2 && chunkRef.at(0).isLetter()
+            && chunkRef.at(1) == u':') {
+            break;
         }
-        return true;
+        const QString chunk = chunkRef.toString();
+        // TODO: get isDirPath() and rmDir() to accept QStringView
+        if (!isDirPath(chunk, nullptr))
+            return false;
+        if (!rmDir(chunk))
+            return oldslash != 0;
+        slash = dirName.lastIndexOf(QDir::separator(), oldslash - 1);
     }
-    return rmDir(entry.filePath());
+
+    return true;
 }
 
 //static
@@ -1770,6 +1788,12 @@ bool QFileSystemEngine::removeFile(const QFileSystemEntry &entry, QSystemError &
     return ret;
 }
 
+//static
+bool QFileSystemEngine::supportsMoveFileToTrash()
+{
+    return true;
+}
+
 /*
     If possible, we use the IFileOperation implementation, which allows us to determine
     the location of the object in the trash.
@@ -1828,12 +1852,10 @@ bool QFileSystemEngine::moveFileToTrash(const QFileSystemEntry &source,
 
 //static
 bool QFileSystemEngine::setPermissions(const QFileSystemEntry &entry,
-                                       QFile::Permissions permissions, QSystemError &error,
-                                       QFileSystemMetaData *data)
+                                       QFile::Permissions permissions, QSystemError &error)
 {
     Q_CHECK_FILE_NAME(entry, false);
 
-    Q_UNUSED(data);
     int mode = 0;
 
     if (permissions & (QFile::ReadOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther))
@@ -1853,6 +1875,12 @@ bool QFileSystemEngine::setPermissions(const QFileSystemEntry &entry,
     return ret;
 }
 
+bool QFileSystemEngine::isCaseSensitive(const QFileSystemEntry &, QFileSystemMetaData &)
+{
+    // FIXME: This may not be accurate for all file systems (QTBUG-28246)
+    return false;
+}
+
 static inline QDateTime fileTimeToQDateTime(const FILETIME *time)
 {
     if (time->dwHighDateTime == 0 && time->dwLowDateTime == 0)
