@@ -14,7 +14,6 @@
 #include <qbytearray.h>
 #include <qdeadlinetimer.h>
 #include <qcoreapplication.h>
-#include <qtimer.h>
 
 #if __has_include(<paths.h>)
 #include <paths.h>
@@ -180,9 +179,7 @@ QProcessEnvironment &QProcessEnvironment::operator=(const QProcessEnvironment &o
 /*!
     \fn void QProcessEnvironment::swap(QProcessEnvironment &other)
     \since 5.0
-
-    Swaps this process environment instance with \a other. This
-    function is very fast and never fails.
+    \memberswap{process environment instance}
 */
 
 /*!
@@ -897,6 +894,16 @@ void QProcessPrivate::Channel::clear()
            if QProcess will actually use \c{vfork(2)} and if \c{vfork(2)} is
            different from standard \c{fork(2)}.
 
+    \value [since 6.9] DisableCoreDumps     Requests that QProcess disable core
+           dumps in the child process. This is useful if the executable being
+           run is likely to crash but users and maintainers are going to be
+           uninterested in generating bug reports for those conditions (for
+           example, the executable is a test process). This setting does not
+           affect the exitStatus() of the crashed process. It is implemented
+           by setting the core dump size resource soft limit to zero, meaning
+           the application can still reverse this change by raising it to a
+           value up to the hard limit.
+
     \sa setUnixProcessParameters(), unixProcessParameters()
 */
 
@@ -1485,22 +1492,22 @@ void QProcess::setStandardInputFile(const QString &fileName)
 
     Redirects the process' standard output to the file \a
     fileName. When the redirection is in place, the standard output
-    read channel is closed: reading from it using read() will always
-    fail, as will readAllStandardOutput().
+    read channel is closed: reading from it using \l read() will always
+    fail, as will \l readAllStandardOutput().
 
-    To discard all standard output from the process, pass nullDevice()
+    To discard all standard output from the process, pass \l nullDevice()
     here. This is more efficient than simply never reading the standard
     output, as no QProcess buffers are filled.
 
-    If the file \a fileName doesn't exist at the moment start() is
+    If the file \a fileName doesn't exist at the moment \l start() is
     called, it will be created. If it cannot be created, the starting
     will fail.
 
-    If the file exists and \a mode is QIODevice::Truncate, the file
-    will be truncated. Otherwise (if \a mode is QIODevice::Append),
+    If the file exists and \a mode is \ QIODeviceBase::Truncate, the file
+    will be truncated. Otherwise (if \a mode is \l QIODeviceBase::Append),
     the file will be appended to.
 
-    Calling setStandardOutputFile() after the process has started has
+    Calling \l setStandardOutputFile() after the process has started has
     no effect.
 
     If \a fileName is an empty string, it stops redirecting the standard
