@@ -5,6 +5,7 @@
 #define QVARIANT_H
 
 #include <QtCore/qatomic.h>
+#include <QtCore/qcompare.h>
 #include <QtCore/qcontainerfwd.h>
 #include <QtCore/qmetatype.h>
 #ifndef QT_NO_DEBUG_STREAM
@@ -123,9 +124,8 @@ public:
         const void *storage() const
         { return is_shared ? data.shared->data() : &data.data; }
 
-        // determine internal storage at compile time
         template<typename T> const T &get() const
-        { return *static_cast<const T *>(CanUseInternalSpace<T> ? &data.data : data.shared->data()); }
+        { return *static_cast<const T *>(storage()); }
 
         inline const QtPrivate::QMetaTypeInterface *typeInterface() const
         {
@@ -496,6 +496,7 @@ public:
         // If possible we reuse the current QVariant private.
         if (isDetached() && d.type() == metaType) {
             *reinterpret_cast<VT *>(const_cast<void *>(constData())) = std::forward<T>(avalue);
+            d.is_null = false;
         } else {
             *this = QVariant::fromValue<VT>(std::forward<T>(avalue));
         }
@@ -550,9 +551,8 @@ public:
             return QVariant();
         QMetaType mt = QMetaType::fromType<Type>();
         mt.registerType(); // we want the type stored in QVariant to always be registered
-        // T is a forwarding reference, so if T satifies the enable-ifery,
-        // we get this overload even if T is an lvalue reference and thus must check here
-        // Moreover, we only try to move if the type is actually moveable and not if T is const
+
+        // We only try to move if the type is actually moveable and not if T is const
         // as in const int i; QVariant::fromValue(std::move(i));
         if constexpr (std::conjunction_v<std::is_move_constructible<Type>, std::negation<std::is_const<T>>>)
             return moveConstruct(QMetaType::fromType<Type>(), std::addressof(value));
@@ -614,10 +614,10 @@ private:
         return std::visit(visitor, std::forward<StdVariant>(v));
     }
 
-    friend inline bool operator==(const QVariant &a, const QVariant &b)
+    friend bool comparesEqual(const QVariant &a, const QVariant &b)
     { return a.equals(b); }
-    friend inline bool operator!=(const QVariant &a, const QVariant &b)
-    { return !a.equals(b); }
+    Q_DECLARE_EQUALITY_COMPARABLE_NON_NOEXCEPT(QVariant)
+
 #ifndef QT_NO_DEBUG_STREAM
     template <typename T>
     friend auto operator<<(const QDebug &debug, const T &variant) -> std::enable_if_t<std::is_same_v<T, QVariant>, QDebug> {
@@ -712,7 +712,7 @@ public:
 
 inline bool QVariant::isValid() const
 {
-    return d.type().isValid();
+    return d.type().isValid(QT6_CALL_NEW_OVERLOAD);
 }
 
 #ifndef QT_NO_DATASTREAM
@@ -770,7 +770,7 @@ template<typename T> inline T qvariant_cast(QVariant &&v)
 {
     QMetaType targetType = QMetaType::fromType<T>();
     if (v.d.type() == targetType) {
-        if constexpr (QVariant::Private::CanUseInternalSpace<T>) {
+        if (!v.d.is_shared) {
             return std::move(*reinterpret_cast<T *>(v.d.data.data));
         } else {
             if (v.d.data.shared->ref.loadRelaxed() == 1)
@@ -795,14 +795,16 @@ template<typename T> inline T qvariant_cast(QVariant &&v)
     return t;
 }
 
+#  ifndef QT_NO_VARIANT
 template<> inline QVariant qvariant_cast<QVariant>(const QVariant &v)
 {
     if (v.metaType().id() == QMetaType::QVariant)
         return *reinterpret_cast<const QVariant *>(v.constData());
     return v;
 }
+#  endif
 
-#endif
+#endif // QT_MOC
 
 #ifndef QT_NO_DEBUG_STREAM
 #if QT_DEPRECATED_SINCE(6, 0)
