@@ -31,6 +31,7 @@ using namespace Qt::StringLiterals;
     \value FilePathRole
     \value FileNameRole
     \value FilePermissions
+    \value FileInfoRole The QFileInfo object for the index
 */
 
 /*!
@@ -54,8 +55,6 @@ using namespace Qt::StringLiterals;
     about the underlying files and directories related to items in the model.
     Directories can be created and removed using mkdir(), rmdir().
 
-    \note QFileSystemModel requires an instance of \l QApplication.
-
     \section1 Example Usage
 
     A directory model that displays the contents of a default directory
@@ -79,13 +78,15 @@ using namespace Qt::StringLiterals;
 
     \section1 Caching and Performance
 
-    QFileSystemModel will not fetch any files or directories until setRootPath()
-    is called.  This will prevent any unnecessary querying on the file system
-    until that point such as listing the drives on Windows.
+    QFileSystemModel uses a separate thread to populate itself, so it will not
+    cause the main thread to hang as the file system is being queried. Calls to
+    rowCount() will return \c 0 until the model populates a directory. The thread
+    in which the QFileSystemModel lives needs to run an event loop to process
+    the incoming data.
 
-    QFileSystemModel uses a separate thread to populate itself so it will not
-    cause the main thread to hang as the file system is being queried.
-    Calls to rowCount() will return 0 until the model populates a directory.
+    QFileSystemModel will not start populating itself until setRootPath() is
+    called. This prevents any unnecessary querying of the system's root file
+    system, such as enumerating the drives on Windows, until that point.
 
     QFileSystemModel keeps a cache with file information. The cache is
     automatically kept up to date using the QFileSystemWatcher.
@@ -309,6 +310,24 @@ static QString qt_GetLongPathName(const QString &strShortPath)
         return QDir::fromNativeSeparators(strShortPath);
     }
 }
+
+static inline void chopSpaceAndDot(QString &element)
+{
+    if (element == "."_L1 || element == ".."_L1)
+        return;
+    // On Windows, "filename    " and "filename" are equivalent and
+    // "filename  .  " and "filename" are equivalent
+    // "filename......." and "filename" are equivalent Task #133928
+    // whereas "filename  .txt" is still "filename  .txt"
+    while (element.endsWith(u'.') || element.endsWith(u' '))
+        element.chop(1);
+
+    // If a file is saved as ' Foo.txt', where the leading character(s)
+    // is an ASCII Space (0x20), it will be saved to the file system as 'Foo.txt'.
+    while (element.startsWith(u' '))
+        element.remove(0, 1);
+}
+
 #endif
 
 /*!
@@ -402,15 +421,10 @@ QFileSystemModelPrivate::QFileSystemNode *QFileSystemModelPrivate::node(const QS
         if (i == pathElements.size() - 1)
             elementPath.append(trailingSeparator);
 #ifdef Q_OS_WIN
-        // On Windows, "filename    " and "filename" are equivalent and
-        // "filename  .  " and "filename" are equivalent
-        // "filename......." and "filename" are equivalent Task #133928
-        // whereas "filename  .txt" is still "filename  .txt"
         // If after stripping the characters there is nothing left then we
         // just return the parent directory as it is assumed that the path
-        // is referring to the parent
-        while (element.endsWith(u'.') || element.endsWith(u' '))
-            element.chop(1);
+        // is referring to the parent.
+        chopSpaceAndDot(element);
         // Only filenames that can't possibly exist will be end up being empty
         if (element.isEmpty())
             return parent;
@@ -730,6 +744,8 @@ QVariant QFileSystemModel::data(const QModelIndex &index, int role) const
         return filePath(index);
     case FileNameRole:
         return d->name(index);
+    case FileInfoRole:
+        return QVariant::fromValue(fileInfo(index));
     case Qt::DecorationRole:
         if (index.column() == QFileSystemModelPrivate::NameColumn) {
             QIcon icon = d->icon(index);
@@ -863,6 +879,12 @@ bool QFileSystemModel::setData(const QModelIndex &idx, const QVariant &value, in
     }
 
     QString newName = value.toString();
+#ifdef Q_OS_WIN
+    chopSpaceAndDot(newName);
+    if (newName.isEmpty())
+        return false;
+#endif
+
     QString oldName = idx.data().toString();
     if (newName == oldName)
         return true;
@@ -998,7 +1020,7 @@ Qt::ItemFlags QFileSystemModel::flags(const QModelIndex &index) const
 /*!
     \internal
 */
-void QFileSystemModelPrivate::_q_performDelayedSort()
+void QFileSystemModelPrivate::performDelayedSort()
 {
     Q_Q(QFileSystemModel);
     q->sort(sortColumn, sortOrder);
@@ -1135,12 +1157,10 @@ void QFileSystemModel::sort(int column, Qt::SortOrder order)
 
     emit layoutAboutToBeChanged();
     QModelIndexList oldList = persistentIndexList();
-    QList<QPair<QFileSystemModelPrivate::QFileSystemNode *, int>> oldNodes;
+    QList<std::pair<QFileSystemModelPrivate::QFileSystemNode *, int>> oldNodes;
     oldNodes.reserve(oldList.size());
-    for (const QModelIndex &oldNode : oldList) {
-        QPair<QFileSystemModelPrivate::QFileSystemNode*, int> pair(d->node(oldNode), oldNode.column());
-        oldNodes.append(pair);
-    }
+    for (const QModelIndex &oldNode : oldList)
+        oldNodes.emplace_back(d->node(oldNode), oldNode.column());
 
     if (!(d->sortColumn == column && d->sortOrder != order && !d->forceSort)) {
         //we sort only from where we are, don't need to sort all the model
@@ -1156,7 +1176,7 @@ void QFileSystemModel::sort(int column, Qt::SortOrder order)
         newList.append(d->index(node, col));
 
     changePersistentIndexList(oldList, newList);
-    emit layoutChanged();
+    emit layoutChanged({}, VerticalSortHint);
 }
 
 /*!
@@ -1255,6 +1275,7 @@ QHash<int, QByteArray> QFileSystemModel::roleNames() const
     ret.insert(QFileSystemModel::FilePathRole, QByteArrayLiteral("filePath"));
     ret.insert(QFileSystemModel::FileNameRole, QByteArrayLiteral("fileName"));
     ret.insert(QFileSystemModel::FilePermissions, QByteArrayLiteral("filePermissions"));
+    ret.insert(QFileSystemModel::FileInfoRole, QByteArrayLiteral("fileInfo"));
     return ret;
 }
 
@@ -1424,17 +1445,24 @@ QModelIndex QFileSystemModel::mkdir(const QModelIndex &parent, const QString &na
     if (!parent.isValid())
         return parent;
 
+    QString fileName = name;
+#ifdef Q_OS_WIN
+    chopSpaceAndDot(fileName);
+    if (fileName.isEmpty())
+        return QModelIndex();
+#endif
+
     QDir dir(filePath(parent));
-    if (!dir.mkdir(name))
+    if (!dir.mkdir(fileName))
         return QModelIndex();
     QFileSystemModelPrivate::QFileSystemNode *parentNode = d->node(parent);
-    d->addNode(parentNode, name, QFileInfo());
-    Q_ASSERT(parentNode->children.contains(name));
-    QFileSystemModelPrivate::QFileSystemNode *node = parentNode->children[name];
+    d->addNode(parentNode, fileName, QFileInfo());
+    Q_ASSERT(parentNode->children.contains(fileName));
+    QFileSystemModelPrivate::QFileSystemNode *node = parentNode->children[fileName];
 #if QT_CONFIG(filesystemwatcher)
-    node->populate(d->fileInfoGatherer->getInfo(QFileInfo(dir.absolutePath() + QDir::separator() + name)));
+    node->populate(d->fileInfoGatherer->getInfo(QFileInfo(dir.absolutePath() + QDir::separator() + fileName)));
 #endif
-    d->addVisibleFiles(parentNode, QStringList(name));
+    d->addVisibleFiles(parentNode, QStringList(fileName));
     return d->index(node);
 }
 
@@ -1459,6 +1487,8 @@ QFile::Permissions QFileSystemModel::permissions(const QModelIndex &index) const
     modify the data available to views. In other words, the "root" of
     the model is \e not changed to include only files and directories
     within the directory specified by \a newPath in the file system.
+
+    \sa {QTreeView::setRootIndex()}, {QtQuick::}{TreeView::rootIndex}
   */
 QModelIndex QFileSystemModel::setRootPath(const QString &newPath)
 {
@@ -1765,7 +1795,7 @@ bool QFileSystemModel::rmdir(const QModelIndex &aindex)
     Performed quick listing and see if any files have been added or removed,
     then fetch more information on visible files.
  */
-void QFileSystemModelPrivate::_q_directoryChanged(const QString &directory, const QStringList &files)
+void QFileSystemModelPrivate::directoryChanged(const QString &directory, const QStringList &files)
 {
     QFileSystemModelPrivate::QFileSystemNode *parentNode = node(directory, false);
     if (parentNode->children.size() == 0)
@@ -1913,7 +1943,7 @@ void QFileSystemModelPrivate::removeVisibleFile(QFileSystemNode *parentNode, int
     The thread has received new information about files,
     update and emit dataChanged if it has actually changed.
  */
-void QFileSystemModelPrivate::_q_fileSystemChanged(const QString &path,
+void QFileSystemModelPrivate::fileSystemChanged(const QString &path,
                                                    const QList<std::pair<QString, QFileInfo>> &updates)
 {
 #if QT_CONFIG(filesystemwatcher)
@@ -1928,6 +1958,11 @@ void QFileSystemModelPrivate::_q_fileSystemChanged(const QString &path,
         QExtendedInformation info = fileInfoGatherer->getInfo(update.second);
         bool previouslyHere = parentNode->children.contains(fileName);
         if (!previouslyHere) {
+#ifdef Q_OS_WIN
+            chopSpaceAndDot(fileName);
+            if (fileName.isEmpty())
+                continue;
+#endif
             addNode(parentNode, fileName, info.fileInfo());
         }
         QFileSystemModelPrivate::QFileSystemNode * node = parentNode->children.value(fileName);
@@ -2024,7 +2059,7 @@ void QFileSystemModelPrivate::_q_fileSystemChanged(const QString &path,
 /*!
     \internal
 */
-void QFileSystemModelPrivate::_q_resolvedName(const QString &fileName, const QString &resolvedName)
+void QFileSystemModelPrivate::resolvedName(const QString &fileName, const QString &resolvedName)
 {
     resolvedSymLinks[fileName] = resolvedName;
 }
@@ -2079,7 +2114,7 @@ QFileSystemModelPrivate::QFileSystemModelPrivate()
 QFileSystemModelPrivate::~QFileSystemModelPrivate()
 {
 #if QT_CONFIG(filesystemwatcher)
-    fileInfoGatherer->requestInterruption();
+    fileInfoGatherer->requestAbort();
     if (!fileInfoGatherer->wait(1000)) {
         // If the thread hangs, perhaps because the network was disconnected
         // while the gatherer was stat'ing a remote file, then don't block
@@ -2097,22 +2132,23 @@ QFileSystemModelPrivate::~QFileSystemModelPrivate()
 */
 void QFileSystemModelPrivate::init()
 {
-    Q_Q(QFileSystemModel);
-
     delayedSortTimer.setSingleShot(true);
 
     qRegisterMetaType<QList<std::pair<QString, QFileInfo>>>();
 #if QT_CONFIG(filesystemwatcher)
-    q->connect(fileInfoGatherer.get(), SIGNAL(newListOfFiles(QString,QStringList)),
-               q, SLOT(_q_directoryChanged(QString,QStringList)));
-    q->connect(fileInfoGatherer.get(), SIGNAL(updates(QString,QList<std::pair<QString,QFileInfo>>)), q,
-               SLOT(_q_fileSystemChanged(QString,QList<std::pair<QString,QFileInfo>>)));
-    q->connect(fileInfoGatherer.get(), SIGNAL(nameResolved(QString,QString)),
-            q, SLOT(_q_resolvedName(QString,QString)));
-    q->connect(fileInfoGatherer.get(), SIGNAL(directoryLoaded(QString)),
-               q, SIGNAL(directoryLoaded(QString)));
+    QObjectPrivate::connect(fileInfoGatherer.get(), &QFileInfoGatherer::newListOfFiles,
+                            this, &QFileSystemModelPrivate::directoryChanged);
+    QObjectPrivate::connect(fileInfoGatherer.get(), &QFileInfoGatherer::updates,
+                            this, &QFileSystemModelPrivate::fileSystemChanged);
+    QObjectPrivate::connect(fileInfoGatherer.get(), &QFileInfoGatherer::nameResolved,
+                            this, &QFileSystemModelPrivate::resolvedName);
+    Q_Q(QFileSystemModel);
+    q->connect(fileInfoGatherer.get(), &QFileInfoGatherer::directoryLoaded,
+               q, &QFileSystemModel::directoryLoaded);
 #endif // filesystemwatcher
-    q->connect(&delayedSortTimer, SIGNAL(timeout()), q, SLOT(_q_performDelayedSort()), Qt::QueuedConnection);
+    QObjectPrivate::connect(&delayedSortTimer, &QTimer::timeout,
+                            this, &QFileSystemModelPrivate::performDelayedSort,
+                            Qt::QueuedConnection);
 }
 
 /*!
