@@ -20,10 +20,12 @@ private slots:
     void construct();
     void constructStream();
     void testSETTINGSFrame();
+    void maxHeaderTableSize();
     void testPING();
     void testRSTServerSide();
     void testRSTClientSide();
     void testRSTReplyOnDATAEND();
+    void resetAfterClose();
     void testBadFrameSize_data();
     void testBadFrameSize();
     void testDataFrameAfterRSTIncoming();
@@ -152,7 +154,8 @@ bool tst_QHttp2Connection::waitForSettingsExchange(QHttp2Connection *client,
     client->handleReadyRead(); // handle incoming frames, send response
 
     bool success = QTest::qWaitFor([&]() {
-        return settingsFrameReceived && serverSettingsFrameReceived;
+        return settingsFrameReceived && serverSettingsFrameReceived
+                && !client->waitingForSettingsACK && !server->waitingForSettingsACK;
     });
 
     disconnect(c);
@@ -211,6 +214,8 @@ void tst_QHttp2Connection::testSETTINGSFrame()
     config.setSessionReceiveWindowSize(SessionReceiveWindowSize);
     auto connection = QHttp2Connection::createDirectConnection(&buffer, config);
     Q_UNUSED(connection);
+    QHttp2Stream *stream = connection->createStream().unwrap();
+    QVERIFY(stream);
     QCOMPARE_GE(buffer.size(), PrefaceLength);
 
     // Preface
@@ -266,12 +271,115 @@ void tst_QHttp2Connection::testSETTINGSFrame()
     }
 }
 
+void tst_QHttp2Connection::maxHeaderTableSize()
+{
+    auto [client, server] = makeFakeConnectedSockets();
+    auto connection = makeHttp2Connection(client.get(), {}, Client);
+    auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
+
+    QSignalSpy incomingRequestSpy(serverConnection, &QHttp2Connection::newIncomingStream);
+
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
+    QVERIFY(waitForSettingsExchange(connection, serverConnection));
+
+    // Test defaults:
+    // encoder:
+    QCOMPARE(connection->encoder.dynamicTableSize(), 0u);
+    QCOMPARE(connection->encoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(connection->encoder.dynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->encoder.dynamicTableSize(), 0u);
+    QCOMPARE(serverConnection->encoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->encoder.dynamicTableCapacity(), 4096u);
+
+    // decoder:
+    QCOMPARE(connection->decoder.dynamicTableSize(), 0u);
+    QCOMPARE(connection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(connection->decoder.dynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->decoder.dynamicTableSize(), 0u);
+    QCOMPARE(serverConnection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->decoder.dynamicTableCapacity(), 4096u);
+
+    // Send a HEADER block with a custom header, to add something to the dynamic table:
+    HPack::HttpHeader headers = getRequiredHeaders();
+    headers.emplace_back("x-test", "test");
+    clientStream->sendHEADERS(headers, true);
+
+    QVERIFY(incomingRequestSpy.wait());
+
+    // Test that the size has been updated:
+    // encoder:
+    QCOMPARE_GT(connection->encoder.dynamicTableSize(), 0u); // now > 0
+    QCOMPARE(connection->encoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(connection->encoder.dynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->encoder.dynamicTableSize(), 0u);
+    QCOMPARE(serverConnection->encoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->encoder.dynamicTableCapacity(), 4096u);
+
+    // decoder:
+    QCOMPARE(connection->decoder.dynamicTableSize(), 0u);
+    QCOMPARE(connection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(connection->decoder.dynamicTableCapacity(), 4096u);
+    QCOMPARE_GT(serverConnection->decoder.dynamicTableSize(), 0u); // now > 0
+    QCOMPARE(serverConnection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->decoder.dynamicTableCapacity(), 4096u);
+
+    const quint32 initialTableSize = connection->encoder.dynamicTableSize();
+    QCOMPARE(initialTableSize, serverConnection->decoder.dynamicTableSize());
+
+    // Notify from server that we want a smaller size, reset to 0 first:
+    for (quint32 nextSize : {0, 2048}) {
+        // @note: currently we don't have an API for this so just do it by hand:
+        serverConnection->waitingForSettingsACK = true;
+        using namespace Http2;
+        FrameWriter builder(FrameType::SETTINGS, FrameFlag::EMPTY, connectionStreamID);
+        builder.append(Settings::HEADER_TABLE_SIZE_ID);
+        builder.append(nextSize);
+        builder.write(*server->out);
+
+        QVERIFY(QTest::qWaitFor([&]() { return !serverConnection->waitingForSettingsACK; }));
+    }
+
+    // Now we have to send another HEADER block without extra field, so we can see that the size of
+    // the dynamic table has decreased after we cleared it:
+    headers = getRequiredHeaders();
+    QHttp2Stream *clientStream2 = connection->createStream().unwrap();
+    clientStream2->sendHEADERS(headers, true);
+
+    QVERIFY(incomingRequestSpy.wait());
+
+    // Test that the size has been updated:
+    // encoder:
+    QCOMPARE_LT(connection->encoder.dynamicTableSize(), initialTableSize);
+
+    QCOMPARE(connection->encoder.maxDynamicTableCapacity(), 2048u);
+    QCOMPARE(connection->encoder.dynamicTableCapacity(), 2048u);
+    QCOMPARE(serverConnection->encoder.dynamicTableSize(), 0u);
+    QCOMPARE(serverConnection->encoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->encoder.dynamicTableCapacity(), 4096u);
+
+    // decoder:
+    QCOMPARE(connection->decoder.dynamicTableSize(), 0u);
+    QCOMPARE(connection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(connection->decoder.dynamicTableCapacity(), 4096u);
+
+    QCOMPARE_LT(serverConnection->decoder.dynamicTableSize(), initialTableSize);
+    // If we add an API for this then this should also be updated at some stage:
+    QCOMPARE(serverConnection->decoder.maxDynamicTableCapacity(), 4096u);
+    QCOMPARE(serverConnection->decoder.dynamicTableCapacity(), 2048u);
+
+    quint32 newTableSize = connection->encoder.dynamicTableSize();
+    QCOMPARE(newTableSize, serverConnection->decoder.dynamicTableSize());
+}
+
 void tst_QHttp2Connection::testPING()
 {
     auto [client, server] = makeFakeConnectedSockets();
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy serverPingSpy{ serverConnection, &QHttp2Connection::pingFrameRecived };
@@ -302,13 +410,13 @@ void tst_QHttp2Connection::testRSTServerSide()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -335,13 +443,13 @@ void tst_QHttp2Connection::testRSTClientSide()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -368,13 +476,13 @@ void tst_QHttp2Connection::testRSTReplyOnDATAEND()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -416,6 +524,46 @@ void tst_QHttp2Connection::testRSTReplyOnDATAEND()
     QCOMPARE(errrorServerSpy.count(), 1);
 }
 
+void tst_QHttp2Connection::resetAfterClose()
+{
+    auto [client, server] = makeFakeConnectedSockets();
+    auto connection = makeHttp2Connection(client.get(), {}, Client);
+    auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
+
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
+    QVERIFY(waitForSettingsExchange(connection, serverConnection));
+
+    QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
+
+    QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
+    HPack::HttpHeader headers = getRequiredHeaders();
+    clientStream->sendHEADERS(headers, true);
+
+    QVERIFY(newIncomingStreamSpy.wait());
+    auto *serverStream = newIncomingStreamSpy.front().front().value<QHttp2Stream *>();
+    QCOMPARE(clientStream->streamID(), serverStream->streamID());
+
+    QSignalSpy errorSpy(clientStream, &QHttp2Stream::errorOccurred);
+
+    const HPack::HttpHeader StatusOKHeaders{ { ":status", "200" } };
+    serverStream->sendHEADERS(StatusOKHeaders, true);
+
+    // Write the RST_STREAM frame manually because we guard against sending RST_STREAM on closed
+    // streams
+    auto &frameWriter = serverConnection->frameWriter;
+    frameWriter.start(Http2::FrameType::RST_STREAM, Http2::FrameFlag::EMPTY,
+                      serverStream->streamID());
+    frameWriter.append(quint32(Http2::Http2Error::STREAM_CLOSED));
+    QVERIFY(frameWriter.write(*serverConnection->getSocket()));
+
+    QVERIFY(clientHeaderReceivedSpy.wait());
+    QCOMPARE(clientStream->state(), QHttp2Stream::State::Closed);
+
+    QTest::qWait(10); // Just needs to process events basically
+    QCOMPARE(errorSpy.count(), 0);
+}
+
 void tst_QHttp2Connection::testBadFrameSize_data()
 {
     QTest::addColumn<uchar>("frametype");
@@ -440,13 +588,13 @@ void tst_QHttp2Connection::testBadFrameSize()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -487,7 +635,7 @@ void tst_QHttp2Connection::testBadFrameSize()
         buffer[2] = size;
 
         auto writtenN = connection->getSocket()->write(reinterpret_cast<const char *>(&buffer[0]), buffer.size());
-        QCOMPARE(writtenN, buffer.size());
+        QCOMPARE(writtenN, qint64(buffer.size()));
     }
 
     QCOMPARE(rstClientSpy.wait(), rst_received);
@@ -501,13 +649,13 @@ void tst_QHttp2Connection::testDataFrameAfterRSTIncoming()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -543,11 +691,11 @@ void tst_QHttp2Connection::testDataFrameAfterRSTOutgoing()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
     QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
@@ -585,14 +733,14 @@ void tst_QHttp2Connection::connectToServer()
     auto connection = makeHttp2Connection(client.get(), {}, Client);
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
     QSignalSpy clientIncomingStreamSpy{ connection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader headers = getRequiredHeaders();
     clientStream->sendHEADERS(headers, false);
 
@@ -619,14 +767,14 @@ void tst_QHttp2Connection::WINDOW_UPDATE()
     config.setStreamReceiveWindowSize(1024); // Small window on server to provoke WINDOW_UPDATE
     auto serverConnection = makeHttp2Connection(server.get(), config, Server);
 
+    QHttp2Stream *clientStream = connection->createStream().unwrap();
+    QVERIFY(clientStream);
     QVERIFY(waitForSettingsExchange(connection, serverConnection));
 
     QSignalSpy newIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream };
 
-    QHttp2Stream *clientStream = connection->createStream().unwrap();
     QSignalSpy clientHeaderReceivedSpy{ clientStream, &QHttp2Stream::headersReceived };
     QSignalSpy clientDataReceivedSpy{ clientStream, &QHttp2Stream::dataReceived };
-    QVERIFY(clientStream);
     HPack::HttpHeader expectedRequestHeaders = HPack::HttpHeader{
         { ":authority", "example.com" },
         { ":method", "POST" },
@@ -644,11 +792,8 @@ void tst_QHttp2Connection::WINDOW_UPDATE()
     // we can check the headers now immediately
     QCOMPARE(serverStream->receivedHeaders(), expectedRequestHeaders);
 
-    QBuffer *buffer = new QBuffer(clientStream);
     QByteArray uploadedData = "Hello World"_ba.repeated(1000);
-    buffer->setData(uploadedData);
-    buffer->open(QIODevice::ReadWrite);
-    clientStream->sendDATA(buffer, true);
+    clientStream->sendDATA(uploadedData, true);
 
     bool streamEnd = false;
     QByteArray serverReceivedData;
@@ -666,10 +811,7 @@ void tst_QHttp2Connection::WINDOW_UPDATE()
 
     const HPack::HttpHeader ExpectedResponseHeaders{ { ":status", "200" } };
     serverStream->sendHEADERS(ExpectedResponseHeaders, false);
-    QBuffer *serverBuffer = new QBuffer(serverStream);
-    serverBuffer->setData(uploadedData);
-    serverBuffer->open(QIODevice::ReadWrite);
-    serverStream->sendDATA(serverBuffer, true);
+    serverStream->sendDATA(uploadedData, true);
 
     QVERIFY(clientHeaderReceivedSpy.wait());
     const HPack::HttpHeader
@@ -753,6 +895,8 @@ void tst_QHttp2Connection::testCONTINUATIONFrame()
     auto [client, server] = makeFakeConnectedSockets(); \
     auto clientConnection = makeHttp2Connection(client.get(), {}, Client); \
     auto serverConnection = makeHttp2Connection(server.get(), {}, Server); \
+    QHttp2Stream *clientStream = clientConnection->createStream().unwrap(); \
+    QVERIFY(clientStream); \
     QVERIFY(waitForSettingsExchange(clientConnection, serverConnection)); \
     \
     HPack::Encoder encoder = HPack::Encoder(HPack::FieldLookupTable::DefaultSize, true); \
@@ -761,8 +905,6 @@ void tst_QHttp2Connection::testCONTINUATIONFrame()
     QSignalSpy serverIncomingStreamSpy{ serverConnection, &QHttp2Connection::newIncomingStream }; \
     QSignalSpy receivedGOAWAYSpy{ clientConnection, &QHttp2Connection::receivedGOAWAY }; \
     \
-    QHttp2Stream *clientStream = clientConnection->createStream().unwrap(); \
-    QVERIFY(clientStream);
 
     // Send multiple CONTINUATION frames
     {
